<?php

/**
 * @file
 * Provides Deepl Pro translator plugin controller.
 *
 * Check @link https://www.deepl.com/api.html Deepl Pro @endlink.
 */

/**
 * Deepl Pro translator plugin controller.
 */
class TMGMTDeeplProTranslatorPluginController extends TMGMTDefaultTranslatorPluginController {

  /**
   * Translation service URL.
   *
   * @var string
   */
  protected $translatorUrl = 'https://api.deepl.com/v2/translate';

  /**
   * Translation usage service URL.
   *
   * @var string
   */
  protected $translatorUsageUrl = 'https://api.deepl.com/v2/usage';

  /**
   * Name of parameter that contains source string to be translated.
   *
   * @var string
   */
  protected $qParamName = 'text';

  /**
   * Max number of text queries for translation sent in one request.
   *
   * @var int
   */
  protected $qChunkSize = 50;

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::isAvailable().
   */
  public function isAvailable(TMGMTTranslator $translator) {
    if ($translator->getSetting('auth_key')) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::canTranslate().
   */
  public function canTranslate(TMGMTTranslator $translator, TMGMTJob $job) {
    if (!parent::canTranslate($translator, $job)) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Implements TMGMTTranslatorPluginControllerInterface::requestTranslation().
   */
  public function requestTranslation(TMGMTJob $job) {
    // Pull the source data array through the job and flatten it.
    $data = array_filter(tmgmt_flatten_data($job->getData()), '_tmgmt_filter_data');

    $translation = array();
    $q = array();
    $keys_sequence = array();
    $i = 0;

    // Build Deepl pro text param and preserve initial array keys.
    foreach ($data as $key => $value) {
      $q[] = $value['#text'];
      $keys_sequence[] = $key;
    }

    try {
      // Split $q into chunks of self::qChunkSize.
      foreach (array_chunk($q, $this->qChunkSize) as $_q) {
        // Get translation from DeepL.
        $result = $this->deeplProRequestTranslation($job, $_q);
        // Collect translated texts with use of initial keys.
        foreach ($result['translations'] as $translated) {
          $translation[$keys_sequence[$i]]['#text'] = $translated['text'];
          $i++;
        }
      }
      // The translation job has been successfully submitted.
      $job->submitted('The translation job has been submitted.');

      // Save the translated data through the job.
      // NOTE that this line of code is reached only in case all translation
      // requests succeeded.
      $job->addTranslatedData(tmgmt_unflatten_data($translation));

      // Check for auto_accept setting in checkout form.
      if ($job->getSetting('auto_accept')) {
        // Get all items and set state to be accepted.
        /** @var \TMGMTJobItem $jobItem */
        foreach ($job->getItems() as $jobItem) {
          $jobItem->acceptTranslation();
        }
      }
    }
    catch (TMGMTDeeplProException $e) {
      $job->rejected('Translation has been rejected with following error: !error',
        array('!error' => $e->getMessage()), 'error');
    }
  }

  /**
   * Helper method to do translation request.
   *
   * @param TMGMTJob $job
   *   TMGMT Job to be used for translation.
   * @param array|string $text
   *   Text/texts to be translated.
   *
   * @return array
   *   Userialized JSON containing translated texts.
   */
  protected function deeplProRequestTranslation(TMGMTJob $job, $text) {
    $translator = $job->getTranslator();
    $remote_mapping = $this->getDefaultRemoteLanguagesMappings();

    $source_language = $remote_mapping[$job->source_language];
    $target_language = $remote_mapping[$job->target_language];

    // Build query params.
    $query_params = array(
      'source_lang' => $source_language,
      'target_lang' => $target_language,
      $this->qParamName => $text,
    );
    return $this->doRequest($translator, $query_params);
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedTargetLanguages(TMGMTTranslator $translator, $source_language) {
    $languages = $this->getDefaultRemoteLanguagesMappings();
    // There are no language pairs, any supported language can be translated
    // into the others. If the source language is part of the languages,
    // then return them all, just remove the source language.
    if (array_key_exists($source_language, $languages)) {
      unset($languages[$source_language]);
      return $languages;
    }

    return array();
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedRemoteLanguages(TMGMTTranslator $translator) {
    // Pre-defined array of available languages.
    return array(
      "EN" => t('English'),
      "DE" => t('German'),
      "FR" => t('French'),
      "ES" => t('Spanish'),
      "IT" => t('Italian'),
      "NL" => t('Dutch'),
      "PL" => t('Polish'),
      "PT" => t('Portuguese'),
      "RU" => t('Russian'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultRemoteLanguagesMappings() {
    return array(
      'en' => 'EN',
      'en-GB' => 'EN',
      'de' => 'DE',
      'fr' => 'FR',
      'es' => 'ES',
      'it' => 'IT',
      'nl' => 'NL',
      'pl' => 'PL',
      'pt-pt' => 'PT',
      'ru' => 'RU',
    );
  }

  /**
   * Local method to do request to DeepL Translate service.
   *
   * @param TMGMTTranslator $translator
   *   The translator entity to get the settings from.
   * @param array $query_params
   *   (Optional) Additional query params to be passed into the request.
   *
   * @return array|object
   *   Unserialized JSON response from DeeplPro.
   *
   * @throws TMGMTDeeplProException.
   *   - Unable to connect to the DeeplPro Service.
   *   - Error returned by the DeeplPro Service.
   */
  protected function doRequest(TMGMTTranslator $translator, array $query_params = array()) {
    // Get custom URL for testing purposes, if available.
    $custom_url = $translator->getSetting('url');
    $url = $custom_url ? $custom_url : $this->translatorUrl;

    // Define headers.
    $headers = array(
      'Content-Type' => 'application/x-www-form-urlencoded',
    );

    // Build the query.
    $query_string = '';
    $query_string .= 'auth_key=' . $translator->getSetting('auth_key');

    // Add text to be translated.
    if (isset($query_params[$this->qParamName])) {
      foreach ($query_params[$this->qParamName] as $source_text) {
        $query_string .= '&text=' . urlencode($source_text);
      }
    }

    // Add source language.
    if (isset($query_params['source_lang'])) {
      $query_string .= '&source_lang=' . $query_params['source_lang'];
    }

    // Add target language.
    if (isset($query_params['target_lang'])) {
      $query_string .= '&target_lang=' . $query_params['target_lang'];
    }

    // Add additional settings.
    if (!empty($translator->getSetting('tag_handling'))) {
      $query_string .= '&tag_handling=' . urlencode($translator->getSetting('tag_handling'));
    }

    if (!empty($translator->getSetting('non_splitting_tags'))) {
      $query_string .= '&non_splitting_tags=' . urlencode($translator->getSetting('non_splitting_tags'));
    }

    if (!empty($translator->getSetting('ignore_tags'))) {
      $query_string .= '&ignore_tags=' . urlencode($translator->getSetting('ignore_tags'));
    }

    // Split sentences/ preserve formatting are set as required options.
    $query_string .= '&split_sentences=' . $translator->getSetting('split_sentences');
    $query_string .= '&preserve_formatting=' . $translator->getSetting('preserve_formatting');

    // Request translation service.
    $response = drupal_http_request($url, array(
      'headers' => $headers,
      'method' => 'POST',
      'data' => $query_string,
    ));

    if ($response->code != 200) {
      throw new TMGMTDeeplProException('Unable to connect to DeeplPro service due to following error: @error at @url',
        array('@error' => $response->error, '@url' => $url));
    }

    // Process the JSON result into array.
    $response = drupal_json_decode($response->data);

    // If we do not have data - we got error.
    if (!isset($response['translations'])) {
      throw new TMGMTDeeplProException('DeeplPro service returned following error: @error',
        array('@error' => $response['error']['message']));
    }
    return $response;
  }

  /**
   * Provide translatorUrl for override in automated testing.
   *
   * @param string $translator_url
   *   Url of translator service.
   */
  final public function setTranslatorUrl($translator_url) {
    $this->translatorUrl = $translator_url;
  }

  /**
   * Return url to usage endpoint - used for account validation.
   *
   * @return string
   *   - Url to usage endpoint.
   */
  public function getTranslatorUsageUrl() {
    return $this->translatorUsageUrl;
  }

}
